@@ -83,8 +83,8 @@ class TestFeature(unittest.TestCase):
         f.txStart = 135646802
         f.txEnd = 135646832
         r = list(f.blat())
-        self.assert_(str(f.txStart) in repr(r))
-        self.assert_(str(f.txEnd) in repr(r))
+        self.assert_(str(f.txStart) in repr(r), r)
+        self.assert_(str(f.txEnd) in repr(r), r)
 
     def test_downstream(self):
         f = self.f
@@ -115,6 +115,14 @@ class TestBasic(unittest.TestCase):
             self.assert_(int(toks[1]) > 10000)
             self.assert_(int(toks[2]) < 20000)
 
+    def test_link(self):
+        feat = self.db.knownGene.first()
+        l = feat.link()
+
+        self.assert_(l == "asdf", l)
+
+
+
     def test_bed_other(self):
         g = self.db
         self.assertEqual(g.cpgIslandExt[12].bed(), 'chr1	829557	830482')
@@ -142,24 +150,75 @@ class TestGene(unittest.TestCase):
         transcript = self.db.refGene.filter_by(name="NM_032129").first()
         self.assertEqual(transcript.bed12(), expected)
 
-class Test2Db(unittest.TestCase):
+class TestDb(unittest.TestCase):
     def setUp(self):
         self.dba = Genome('hg18', host="localhost", user="brentp")
-        self.dbb = Genome('hg19', host="localhost", user="brentp")
+        #self.dbb = Genome('hg19', host="localhost", user="brentp")
+
+    def test_protein(self):
+
+        g = self.dba.knownGene.filter_by(name="uc010ntk.1").first()
+        prot = g.protein
+
+        self.assert_(prot.startswith("MIITQTSHCYMTSLGILFLINILPGTTGQGESRRQEPGDFVKQDIG"), prot)
+        self.assert_(prot.endswith("SAIKGMIRKQ"), prot)
 
     def test_ok(self):
         ga = self.dba.refGene.filter_by(name2="MUC5B").first()
-        gb = self.dbb.refGene.filter_by(name2="MUC5B").first()
-        assert ga.txStart != gb.txStart
-        # TODO: these shouldb e different, but they both hg18
-        # session stuff?
-        print ga.db, gb.db
+        self.assert_(ga is not None)
 
     def test_bins(self):
         bins = Genome.bins(12345, 56779)
-        expected = set([1, 9, 73, 585])
+        expected = set([0, 1, 9, 73, 585])
         self.assertEqual(bins, expected)
 
+    def test_tables(self):
+        self.assert_("refGene" in self.dba.tables)
+
+    def test_nearest(self):
+        from cruzdb.models import Feature
+        f = Feature()
+        f.chrom = "chr1"
+        f.txStart = 10
+        f.txEnd = 61
+        #db = Genome('hg18', host="localhost", user="brentp")
+        db = self.dba
+        self.assert_(db.refGene.first() is not None)
+        self.assert_(db.refGene is not None)
+
+        res = db.knearest(db.refGene, f, k=2)
+        self.assert_(len(res) >= 2)
+
+        f = db.refGene.first()
+        key = (f.chrom, f.start, f.end, f.name)
+
+        for k in (2, 4, 6):
+            res = db.knearest("refGene", f, k=k)
+            assert len(res) >= k
+            self.assert_(key in ((n.chrom, n.start, n.end, n.name) for n in res),
+                    (res, f))
+
+
+        f = db.refGene.order_by(db.refGene.table().c.txStart).filter(db.refGene.table().c.strand == "+").first()
+        assert f in db.upstream(db.refGene, f)
+
+        down = db.downstream(db.refGene, f, k=10)
+        self.assert_(len(down) >= 10)
+
+        self.assert_(all(d.start >= f.start for d in down))
+
+
+    def test_down_neg(self):
+        db = self.dba
+        fm = db.refGene.filter(db.refGene.table().c.strand == "-").first()
+        down = db.downstream(db.refGene, fm, k=10)
+
+        self.assert_(all(d.start <= fm.start for d in down))
+
+    def test_dataframe(self):
+        kg = Genome('hg18', host='localhost').dataframe('knownGene', limit=10)
+        assert kg.shape[0] == 10
+
     def test_mirror(self):
 
         try:
@@ -176,6 +235,8 @@ class Test2Db(unittest.TestCase):
         self.assertEqual(a, b)
         os.unlink('/tmp/__u.db')
 
+    def tearDown(self):
+        del self.dba
 
 if __name__ == "__main__":
     unittest.main()
